using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using WcfServices.Dto;

namespace WcfServices.Contract
{
    [ServiceContract]
    public interface ITimeSheetRecordService
    {

        [OperationContract]
        Task<IEnumerable<TimeSheetRecordDto>> GetTimeSheetRecordsAsync(FilterDto filterDto);
        [OperationContract]
        Task GenerateTimeSheetRecordsAsync(int employeeId, int year, int month);
        [OperationContract]
        Task<TimeSheetRecordDto> GetTimeSheetRecordByIdAsync(int id);
        [OperationContract]
        Task CreateTimeSheetRecordAsync(TimeSheetRecordDto userDto);
        [OperationContract]
        Task UpdateTimeSheetRecordAsync(TimeSheetRecordDto userDto);
        [OperationContract]
        Task DeleteTimeSheetRecordAsync(int id);

    }

}
